//
// Created by trotfunky on 06/06/19.
//

#ifndef SRC_LEVEL_H
#define SRC_LEVEL_H

#include <SFML/Graphics.hpp>
#include <pugixml.hpp>
#include <vector>
#include <cstring>
#include <set>
#include <map>
#include <memory>

#include "Utils.h"
#include "Entity.h"
#include "Rule.h"


class Level {
public:
    Level(const pugi::xml_document& xmlDoc, const pro_maat::TextureStore& textureStore);

    /// Add a new rule on top of existing ones, thus with a lower priority
    ///
    /// \param affectedEntities Entities to which the rule will be applied
    /// \param targetState      What action will the rule perform
    /// \param targetEntities   Which entities will be targeted
    void addRule(EntityType affectedEntities, State targetState, EntityType targetEntities);

    void render(sf::RenderWindow& renderWindow) const;
    void runStep();

private:
    const pro_maat::GridPos size;

    std::vector<std::shared_ptr<Entity>> entities;

    const pro_maat::TextureStore& textures;

    //
    // Pathfinding
    //

    std::vector<pro_maat::GridPos> occupiedSquares;

    /// If sign is +1, finds a path between start and goal positions.
    /// If sign is -1, finds a path away from the goal position, from the starting position.
    /// THIS DOES NOT HANDLE ENTITIES BIGGER THAN ONE SQUARE
    ///
    /// \param start Starting position
    /// \param goal  Goal position
    /// \param sign  +1 or -1, defines if we go towards or away from the goal
    /// \return      Orientation of the next move if successful, Orientation::None otherwise.
    Orientation findPath(pro_maat::GridPos start, pro_maat::GridPos goal, int sign);
};


#endif //SRC_LEVEL_H
