//
// Created by trotfunky on 07/06/19.
//

#include "Utils.h"

namespace pro_maat
{
    void errorWindow(const std::string& error)
    {
        sf::Font font;
        if(!font.loadFromFile(std::string(pro_maat::fontFolder)+"verdana.ttf"))
        {
            std::cerr << "Error while loading font for error :" << std::endl;
            std::cerr << error << std::endl;
        }

        sf::Text errorMessage(error,font);
        errorMessage.setFillColor(sf::Color::Red);
        sf::Rect errorArea = errorMessage.getGlobalBounds();
        errorMessage.setPosition(errorArea.width*(1.5/2-0.5),errorArea.height*(0.5));
        sf::RenderWindow window(sf::VideoMode(errorArea.width*1.5,errorArea.height*2),"ERROR");
        while (window.isOpen())
        {
            sf::Event event;
            while(window.pollEvent(event))
            {
                if (event.type == sf::Event::Closed || event.type == sf::Event::KeyPressed || event.type == sf::Event::MouseButtonPressed)
                {
                    window.close();
                    exit(-1);
                }
            }
            window.clear(sf::Color::White);
            window.draw(errorMessage);
            window.display();
        }
    }

    bool isInMap(const GridPos& square, const GridPos& gridSize)
    {
        return !(square.first < 0 || square.second < 0 ||
                square.first > gridSize.first || square.second > gridSize.second);
    }

    float manhattanDistance(const GridPos& leftHandSide, const GridPos& rightHandSide)
    {
        // The *0.01 helps with breaking ties and minimizing exploration
        // As per http://theory.stanford.edu/~amitp/GameProgramming/Heuristics.html#breaking-ties
        return (std::abs(rightHandSide.first-leftHandSide.first)+
                std::abs(rightHandSide.second-leftHandSide.second))*1.01;
    }

    const std::vector<pro_maat::GridPos> getNeighbours(const pro_maat::GridPos& origin, const GridPos& gridSize)
    {
        std::vector<pro_maat::GridPos> neighbours{};
        neighbours.reserve(4);

        for(GridUnit i = std::max(origin.first-1,0);i<=std::min(static_cast<GridUnit>(origin.first+1),gridSize.first);i++)
        {
            // Do not add the original position
            if(i == origin.first) continue;

            neighbours.emplace_back(i,origin.second);
        }
        for(GridUnit i = std::max(origin.second-1,0);i<=std::min(static_cast<GridUnit>(origin.second+1),gridSize.second);i++)
        {
            if(i == origin.second) continue;

            neighbours.emplace_back(origin.first,i);
        }

        return std::move(neighbours);
    }
}